package com.migao.config.response;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * 构建统一响应格式
 *
 * @param <T>
 */

public class ResponseBuilder<T> {

    /**
     * 相应单一对象
     *
     * @param
     * @param
     * @return
     */
    public static <T> ResponseBean<T> ok() {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }

    /**
     * 相应单一对象
     *
     * @param
     * @param
     * @return
     */
    public static <T> ResponseBean<T> ok(Integer status, String message) {
        return ResponseBean.<T>builder().status(status).message(message).build();
    }

    /**
     * 相应单一对象
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> ResponseBean<T> ok(T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).data(data).build();
    }

    /**
     * 相应单一对象
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> ResponseBean<T> ok(T data, String message) {
        return ResponseBean.<T>builder().status(ResponseEnum.OK.status).message(message).data(data).build();
    }


    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<List<R>> ok(List<T> dataList, Function<? super T, ? extends R> function) {
        return ResponseBean.<List<R>>builder().data(buildBean(dataList, function)).status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> ResponseBean<PageBean<R>> ok(Page<T> page, Function<? super T, ? extends R> function) {
        return ResponseBean.<PageBean<R>>builder().data(buildPageBean(page, function)).status(ResponseEnum.OK.status).message(ResponseEnum.OK.message).build();
    }


    /**
     * 构建分页
     *
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> List<R> buildBean(List<T> dataList, Function<? super T, ? extends R> function) {
        return dataList.stream().map(function).collect(Collectors.toList());
    }

    /**
     * 构建分页
     *
     * @param page
     * @param function
     * @param <T>
     * @param <R>
     * @return
     */
    public static <T, R> PageBean<R> buildPageBean(Page<T> page, Function<? super T, ? extends R> function) {
        return PageBean
                .<R>builder()
                .row(
                        page.getRecords().stream().map(function).collect(Collectors.toList())
                )
                .total(page.getTotal())
                .page(page.getPages())
                .build();
    }

    /**
     * 失败
     *
     * @return
     */
    public static <T> ResponseBean<T> fail() {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(ResponseEnum.FAIL.message).build();
    }

    /**
     * 失败
     */
    public static <T> ResponseBean<T> fail(Integer status, String message) {
        return ResponseBean.<T>builder().status(status).message(message).build();
    }

    /**
     * 失败
     */
    public static <T> ResponseBean<T> fail(String message) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(message).build();
    }

    /**
     * 失败
     *
     * @return
     */
    public static <T> ResponseBean<T> fail(T data) {
        return ResponseBean.<T>builder().status(ResponseEnum.FAIL.status).message(ResponseEnum.FAIL.message).data(data).build();
    }

    public static <T> ResponseBean<T> fail(ResponseEnum responseEnum) {
        return ResponseBean.<T>builder().status(responseEnum.status).message(responseEnum.message).build();
    }

}
