package com.migao.controller;


import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.validation.Insert;
import com.migao.config.validation.Update;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.MilestoneQueryRes;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.service.MilestoneService;

import com.migao.service.PurchasingApprovalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.util.List;

@Api(tags = "004.里程碑")
@RestController
@RequestMapping("/milestone")
public class MilestoneController {

    @Resource
    private MilestoneService milestoneService;

    @Resource
    private PurchasingApprovalService purchasingApprovalService;

    @ApiOperationSupport(order = 2)
    @ApiOperation("新增")
    @PostMapping(value = "/insert")
    public ResponseBean<?> insert(
            @Validated(value = {Insert.class}) @RequestBody MilestoneInsertReq milestoneInsertReq
    ) {
        return milestoneService.insert(milestoneInsertReq);
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation("删除")
    @PostMapping(value = "/deleteById")
    public ResponseBean<?> deleteById(
            @RequestParam Integer id
    ) {
        return milestoneService.deleteById(id);
    }

    @ApiOperationSupport(order = 4)
    @ApiOperation("批量删除")
    @PostMapping(value = "/deleteBatch")
    public ResponseBean<?> deleteBatch(
            @RequestBody List<Integer> ids
    ) {
        return milestoneService.deleteBatch(ids);
    }

    @ApiOperationSupport(order = 5)
    @ApiOperation("修改")
    @PostMapping(value = "/update")
    public ResponseBean<?> update(
            @Validated(value = {Update.class}) @RequestBody MilestoneUpdateReq milestoneUpdateReq
    ) {
        return milestoneService.update(milestoneUpdateReq);
    }

    @ApiOperationSupport(order = 1)
    @ApiOperation("分页查询(所有子项目)")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(
            @RequestBody PurchasingApprovalPageQueryReq purchasingApprovalPageQueryReq
    ) {
        return purchasingApprovalService.pageQuery(purchasingApprovalPageQueryReq);
    }


    @ApiOperationSupport(order = 8)
    @ApiOperation("导入")
    @PostMapping(value = "/importData")
    public ResponseBean<?> importData(
            @RequestParam("fileId") String fileId
    ) {
        return milestoneService.importData(fileId);
    }

    @ApiOperationSupport(order = 9)
    @ApiOperation("导出")
    @PostMapping(value = "/export")
    public ResponseBean<?> export(
            @RequestBody MilestonePageQueryReq milestonePageQueryReq
    ) {
        return milestoneService.export(milestonePageQueryReq);
    }
}
