package com.migao.controller;


import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.validation.Insert;
import com.migao.config.validation.Update;
import com.migao.entity.vo.req.ProjectPageQueryReq;
import com.migao.entity.vo.req.ProjectUpdateReq;
import com.migao.entity.vo.req.SubProjectInsertReq;
import com.migao.entity.vo.res.DictionaryListRes;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.service.ProjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Api(tags = "004.储备下达")
@RestController
@RequestMapping("/project/give")
public class ProjectControlController {

    @Resource
    private ProjectService projectService;

    @ApiOperationSupport(order = 1)
    @ApiOperation("分页查")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBean<ProjectQueryRes>> pageQueryControl(
            @RequestBody ProjectPageQueryReq projectPageQueryReq
    ) {
        return projectService.pageQueryControl(projectPageQueryReq);
    }

    @ApiOperationSupport(order = 2)
    @ApiOperation("新增子项目")
    @PostMapping(value = "/insert")
    public ResponseBean<?> insertSubProject(
            @Validated(value = {Insert.class}) @RequestBody SubProjectInsertReq subProjectInsertReq
    ) {
        return projectService.insertSubProject(subProjectInsertReq);
    }

    @ApiOperation("项目分类下拉列表")
    @PostMapping(value = "/projectClassify")
    public ResponseBean<List<DictionaryListRes>> projectClassify() {
        return projectService.projectClassify();
    }

    @ApiOperation("项目专业分类下拉列表")
    @PostMapping(value = "/specialtyClassify")
    public ResponseBean<List<DictionaryListRes>> specialtyClassify() {
        return projectService.specialtyClassify();
    }

    @ApiOperation("二级分类下拉列表")
    @PostMapping(value = "/twoClassify")
    public ResponseBean<List<DictionaryListRes>> twoClassify() {
        return projectService.twoClassify();
    }

    @ApiOperationSupport(order = 6)
    @ApiOperation("删")
    @PostMapping(value = "/deleteById")
    public ResponseBean<?> deleteById(
            @RequestParam Integer id
    ) {
        return projectService.deleteById(id);
    }


    @ApiOperationSupport(order = 5)
    @ApiOperation("改")
    @PostMapping(value = "/update")
    public ResponseBean<?> update(
            @Validated(value = {Update.class}) @RequestBody ProjectUpdateReq projectUpdateReq
    ) {
        return projectService.update(projectUpdateReq);
    }

    @ApiOperationSupport(order = 7)
    @ApiOperation("根据地查询详情")
    @PostMapping(value = "/findById")
    public ResponseBean<ProjectQueryRes> findById(
            @RequestParam Integer id
    ) {
        return projectService.findById(id);
    }


    @ApiOperationSupport(order = 9)
    @ApiOperation("导出")
    @PostMapping(value = "/export")
    public ResponseBean<?> export(
            @RequestBody ProjectPageQueryReq projectPageQueryReq
    ) {
        return projectService.export(projectPageQueryReq);
    }

}
