package com.migao.controller;


import com.migao.config.constant.ExamineStatus;
import com.migao.config.constant.FinancesStatus;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.validation.Insert;
import com.migao.config.validation.Update;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.DictionaryListRes;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.service.ProjectService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Api(tags = "003.項目储备")
@RestController
@RequestMapping("/project/store")
public class ProjectStoreController {

    @Resource
    private ProjectService projectService;


    @ApiOperationSupport(order = 2)
    @ApiOperation("新增储备")
    @PostMapping(value = "/insert")
    public ResponseBean<?> insert(
            @Validated(value = {Insert.class}) @RequestBody ProjectInsertReq projectInsertReq
    ) {
        return projectService.insert(projectInsertReq);
    }

    @ApiOperation("项目分类下拉列表")
    @PostMapping(value = "/projectClassify")
    public ResponseBean<List<DictionaryListRes>> projectClassify() {
        return projectService.projectClassify();
    }

    @ApiOperation("项目专业分类下拉列表")
    @PostMapping(value = "/specialtyClassify")
    public ResponseBean<List<DictionaryListRes>> specialtyClassify() {
        return projectService.specialtyClassify();
    }

    @ApiOperation("二级分类下拉列表")
    @PostMapping(value = "/twoClassify")
    public ResponseBean<List<DictionaryListRes>> twoClassify() {
        return projectService.twoClassify();
    }

    @ApiOperationSupport(order = 6)
    @ApiOperation("删")
    @PostMapping(value = "/deleteById")
    public ResponseBean<?> deleteById(
            @RequestParam Integer id
    ) {
        return projectService.deleteById(id);
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation("审批通过PASS/拒绝TURN")
    @PostMapping(value = "/approve")
    public ResponseBean<?> approve(
            @RequestParam Integer id,
            @ApiParam("审批通过PASS/拒绝TURN") @RequestParam ExamineStatus status
    ) {
        return projectService.approve(id,status);
    }


    @ApiOperation("批量删")
    @PostMapping(value = "/deleteBatch")
    public ResponseBean<?> deleteBatch(
            @RequestBody List<Integer> ids
    ) {
        return projectService.deleteBatch(ids);
    }

    @ApiOperationSupport(order = 5)
    @ApiOperation("改")
    @PostMapping(value = "/update")
    public ResponseBean<?> update(
            @Validated(value = {Update.class}) @RequestBody ProjectUpdateReq projectUpdateReq
    ) {
        return projectService.update(projectUpdateReq);
    }


    @ApiOperationSupport(order = 1)
    @ApiOperation("分页查")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(
            @RequestBody ProjectPageQueryReq projectPageQueryReq
    ) {
        return projectService.pageQuery(projectPageQueryReq);
    }

    @ApiOperationSupport(order = 7)
    @ApiOperation("根据地查询详情")
    @PostMapping(value = "/findById")
    public ResponseBean<ProjectQueryRes> findById(
            @RequestParam Integer id
    ) {
        return projectService.findById(id);
    }

    @ApiOperationSupport(order = 8)
    @ApiOperation("导入")
    @PostMapping(value = "/importData")
    public ResponseBean<?> importData(
            @RequestParam("fileId") String fileId
    ) {
        return projectService.importData(fileId);
    }

    @ApiOperationSupport(order = 9)
    @ApiOperation("导出")
    @PostMapping(value = "/export")
    public ResponseBean<?> export(
            @RequestBody ProjectPageQueryReq projectPageQueryReq
    ) {
        return projectService.export(projectPageQueryReq);
    }

}
