package com.migao.controller;


import com.alibaba.excel.EasyExcel;
import com.migao.config.authority.LoginUtils;
import com.migao.config.listener.ExcelListener;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.Supplier;
import com.migao.entity.po.User;
import com.migao.entity.vo.req.SupplierExcelReq;
import com.migao.service.SupplierService;
import com.migao.util.EntityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;

/**
 * @author dingsong
 */
@Api(tags = "004.供应商模快")
@RestController
@RequestMapping("/supplier")
public class SupplierController {
    @Resource
    SupplierService SupplierService;

    @ApiOperation("导入供应商信息")
    @PostMapping(value = "/importSupplier")
    public ResponseBean importSupplier(@RequestPart MultipartFile file) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        List<SupplierExcelReq> supplierExcelList = new ArrayList<>();
        try {
            EasyExcel.read(file.getInputStream(), SupplierExcelReq.class, new ExcelListener<>(supplierExcelList)).sheet().doRead();
        } catch (IOException e) {
            e.printStackTrace();
        }

        supplierExcelList.forEach(
                x -> {
                    Supplier supplier = EntityUtils.copyProperties(x,Supplier.class);
                    supplier.setLicenseNumber(Integer.parseInt(x.getLicenseNumber()));
                    supplier.setPhoneOne(Integer.parseInt(x.getPhoneOne()));
                    supplier.setPhoneTwo(Integer.parseInt(x.getPhoneTwo()));
                    supplier.setPhone(Integer.parseInt(x.getPhone()));
                    supplier.setCompanyCreateTime(x.getCompanyCreateTime().toInstant()
                            .atZone(ZoneId.systemDefault()).toLocalDateTime());
                    supplier.setDeleted(0);
                    supplier.setCreateTime(LocalDateTime.now());
                    supplier.setCreateUserId(user.getId());
                    SupplierService.save(supplier);
                }
        );

        return ResponseBuilder.ok(200,"导入成功");
    }
}
