package com.migao.controller;

import com.migao.config.log.LogAnnotation;
import com.migao.config.properties.SystemProperties;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.validation.Insert;
import com.migao.entity.vo.req.ProjectTypePageQueryReq;
import com.migao.entity.vo.req.UserInsertReq;
import com.migao.entity.vo.req.UserLoginReq;
import com.migao.entity.vo.req.UserPageQueryReq;
import com.migao.entity.vo.res.ProjectTypeQueryRes;
import com.migao.entity.vo.res.UserQueryRes;
import com.migao.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Api(tags = "001.用户")
@RestController
@RequestMapping("/user")
public class UserController {

    @Resource
    private UserService userService;
    @Resource
    private SystemProperties systemProperties;

    @ApiOperationSupport(order =2)
    @ApiOperation("新增用户")
    @PostMapping(value = "/insert")
    @LogAnnotation(describe = "新增用户")
    public ResponseBean<?> insert(
            @Validated(value = {Insert.class}) @RequestBody UserInsertReq userInsertReq
    ) {
        return userService.insert(userInsertReq);
    }

    @ApiOperationSupport(order =3)
    @ApiOperation("登录")
    @PostMapping(value = "/login")
    @LogAnnotation(describe = "登录",isParameters = "否")
    public ResponseEntity<?> login(@RequestBody UserLoginReq userLoginReq) {
        return userService.login(userLoginReq);
    }

    @ApiOperationSupport(order =1)
    @ApiOperation("分页查")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBean<UserQueryRes>> pageQuery(
            @RequestBody UserPageQueryReq userPageQueryReq
    ) {
        return userService.pageQuery(userPageQueryReq);
    }
}
