package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.migao.config.authority.LoginUtils;
import com.migao.config.response.*;
import com.migao.entity.po.DemandAttachment;
import com.migao.entity.po.DemandLibrary;
import com.migao.entity.po.User;
import com.migao.entity.vo.req.DemandLibraryInsertReq;
import com.migao.entity.vo.req.DemandLibraryQueryReq;
import com.migao.entity.vo.req.DemandLibraryUpdateReq;
import com.migao.entity.vo.req.DemandSubProjectReq;
import com.migao.entity.vo.res.DemandAttachmentRes;
import com.migao.entity.vo.res.DemandLibraryPageQueryRes;
import com.migao.entity.vo.res.DemandLibraryQueryRes;
import com.migao.mapper.*;
import com.migao.service.DemandLibraryService;
import com.migao.util.EntityUtils;
import com.migao.util.ProjectNoUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author dingsong
 */
@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class DemandLibraryServiceImpl extends ServiceImpl<DemandLibraryMapper, DemandLibrary> implements DemandLibraryService {
    @Resource
    DemandLibraryMapper demandLibraryMap;
    @Resource
    DemandAttachmentMapper demandAttachmentMapper;
    @Resource
    DictionaryMapper dictionaryMapper;
    @Resource
    ProjectMapper projectMapper;
    @Resource
    FileInfoMapper fileInfoMapper;
    @Resource
    UserMapper userMapper;

    @Override
    public ResponseBean<?> insert(DemandLibraryInsertReq demandLibrary) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        //提交主需求
        DemandLibrary dic = EntityUtils.copyProperties(demandLibrary,DemandLibrary.class);
        dic.setSubmitTime(LocalDateTime.now());
        dic.setDeleted(0);
        dic.setCreateUserId(user.getId());
        dic.setDemandNumber(ProjectNoUtil.getCode());
        demandLibraryMap.insert(dic);

        //更新子需求父id
        for(Integer i : demandLibrary.getSubProjectId()){
            DemandLibrary dicn = demandLibraryMap.selectById(i);
            dicn.setSubProjectId(dic.getId());
            demandLibraryMap.updateById(dicn);
        }

        //提交附件
        for(Integer i : demandLibrary.getFileId()){
            DemandAttachment demandAttachment = DemandAttachment
                    .builder()
                    .demandId(dic.getId())
                    .fileId(i)
                    .deleted(0)
                    .createTime(LocalDateTime.now())
                    .createUserId(user.getId())
                    .build();
            demandAttachmentMapper.insert(demandAttachment);
        }

        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> insertSubProject(DemandSubProjectReq demandSubProjectReq) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();
        //提交子需求
        DemandLibrary dic = EntityUtils.copyProperties(demandSubProjectReq,DemandLibrary.class);
        dic.setSubmitTime(LocalDateTime.now());
        dic.setDeleted(0);
        dic.setCreateUserId(user.getId());
        dic.setDemandNumber(ProjectNoUtil.getCode());
        demandLibraryMap.insert(dic);
        return ResponseBuilder.ok(dic.getId());
    }

    @Override
    public ResponseBean<?> deleteById(String id) {
        Integer idInt = Integer.parseInt(id);
        DemandLibrary dic = demandLibraryMap.selectById(idInt);
        dic.setDeleted(1);
        demandLibraryMap.updateById(dic);
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> update(DemandLibraryUpdateReq libraryUpdateReq) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        //更新主需求
        DemandLibrary dic = EntityUtils.copyProperties(libraryUpdateReq,DemandLibrary.class);
        dic.setSubmitTime(LocalDateTime.now());
        demandLibraryMap.updateById(dic);

        //更新子需求
        for(Integer i : libraryUpdateReq.getSubProjectId()){
            DemandLibrary dicn = demandLibraryMap.selectById(i);
            dicn.setSubProjectId(dic.getId());
            demandLibraryMap.updateById(dicn);
        }

        //更新附件
        for(Integer i : libraryUpdateReq.getFileId()){
            DemandAttachment demandAttachment = DemandAttachment
                    .builder()
                    .demandId(dic.getId())
                    .fileId(i)
                    .deleted(0)
                    .createTime(LocalDateTime.now())
                    .createUserId(user.getId())
                    .build();
            demandAttachmentMapper.insert(demandAttachment);
        }
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQuery(BasePageRequest basePageRequest) {
        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandLibrary> dynamicQuery = Wrappers.<DemandLibrary>lambdaQuery();
        dynamicQuery.isNull(DemandLibrary::getSubProjectId);
        dynamicQuery.eq(DemandLibrary::getDeleted,0);
        dynamicQuery.orderByDesc(DemandLibrary::getSubmitTime);

        Page<DemandLibrary> all = demandLibraryMap.selectPage(new Page<>(page, size),dynamicQuery);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandLibraryPageQueryRes dicPageQueryRes = DemandLibraryPageQueryRes
                            .builder()
                            .id(element.getId())
                            .demandNumber(element.getDemandNumber())
                            .demandName(element.getDemandName())
                            .demandContent(element.getDemandContent())
                            .demandClass(dictionaryMapper.selectById(element.getDemandClassId()).getValue())
                            .unitName(element.getUnitName())
                            .demandStatus(element.getDemandStatus() == -1 ? "废除"
                                    : element.getDemandStatus() == 0 ? "立项" : "认领")
                            .demandSchedule(element.getDemandSchedule())
                            .associatedProjectId(projectMapper.selectById(element.getAssociatedProjectId()).getName())
                            .submitter(element.getSubmitter())
                            .submitTime(element.getSubmitTime())
                            .deleted(element.getDeleted())
                            .build();
                            return dicPageQueryRes;
                    }
                );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryById(BasePageRequest basePageRequest) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandLibrary> dynamicQuery = Wrappers.<DemandLibrary>lambdaQuery();
        dynamicQuery.isNull(DemandLibrary::getSubProjectId);
        dynamicQuery.eq(DemandLibrary::getDeleted,0);
        if(user.getId() != null) {
            dynamicQuery.eq(DemandLibrary::getCreateUserId, user.getId());
        }
        dynamicQuery.orderByDesc(DemandLibrary::getSubmitTime);

        Page<DemandLibrary> all = demandLibraryMap.selectPage(new Page<>(page, size),dynamicQuery);
        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandLibraryPageQueryRes dicPageQueryRes = DemandLibraryPageQueryRes
                            .builder()
                            .id(element.getId())
                            .demandNumber(element.getDemandNumber())
                            .demandName(element.getDemandName())
                            .demandContent(element.getDemandContent())
                            .demandClass(dictionaryMapper.selectById(element.getDemandClassId()).getValue())
                            .unitName(element.getUnitName())
                            .demandStatus(element.getDemandStatus() == -1 ? "废除"
                                    : element.getDemandStatus() == 0 ? "立项" : "认领")
                            .demandSchedule(element.getDemandSchedule())
                            .associatedProjectId(projectMapper.selectById(element.getAssociatedProjectId()).getName())
                            .submitter(element.getSubmitter())
                            .submitTime(element.getSubmitTime())
                            .deleted(element.getDeleted())
                            .build();
                    return dicPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryOne(DemandLibraryQueryReq demandLibraryQuery) {
        Integer page = demandLibraryQuery.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = demandLibraryQuery.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandLibrary> dynamicQuery = Wrappers.<DemandLibrary>lambdaQuery();
        dynamicQuery.isNull(DemandLibrary::getSubProjectId);
        dynamicQuery.eq(DemandLibrary::getDeleted,0);
        if(StringUtils.isNotBlank(demandLibraryQuery.getKey())){
            //需求编号
            dynamicQuery.like(DemandLibrary::getDemandNumber,demandLibraryQuery.getKey());
            //需求名称
            LambdaQueryWrapper<DemandLibrary> or1 = dynamicQuery.or();
            or1.like(DemandLibrary::getDemandName,demandLibraryQuery.getKey());
            //需求内容
            LambdaQueryWrapper<DemandLibrary> or2 = or1.or();
            or2.like(DemandLibrary::getDemandContent,demandLibraryQuery.getKey());
            //提交单位
            LambdaQueryWrapper<DemandLibrary> or3 = or2.or();
            or3.like(DemandLibrary::getUnitName,demandLibraryQuery.getKey());
            //提交人
            LambdaQueryWrapper<DemandLibrary> or4 = or3.or();
            or4.like(DemandLibrary::getSubmitter,demandLibraryQuery.getKey());
        }

        dynamicQuery.orderByDesc(DemandLibrary::getSubmitTime);

        Page<DemandLibrary> all = demandLibraryMap.selectPage(new Page<>(page, size),dynamicQuery);
        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandLibraryPageQueryRes dicPageQueryRes = DemandLibraryPageQueryRes
                            .builder()
                            .id(element.getId())
                            .demandNumber(element.getDemandNumber())
                            .demandName(element.getDemandName())
                            .demandContent(element.getDemandContent())
                            .demandClass(dictionaryMapper.selectById(element.getDemandClassId()).getValue())
                            .unitName(element.getUnitName())
                            .demandStatus(element.getDemandStatus() == -1 ? "废除"
                                    : element.getDemandStatus() == 0 ? "立项" : "认领")
                            .demandSchedule(element.getDemandSchedule())
                            .associatedProjectId(projectMapper.selectById(element.getAssociatedProjectId()).getName())
                            .submitter(element.getSubmitter())
                            .submitTime(element.getSubmitTime())
                            .deleted(element.getDeleted())
                            .build();
                    return dicPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryOneById(DemandLibraryQueryReq demandLibraryQuery) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        Integer page = demandLibraryQuery.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = demandLibraryQuery.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandLibrary> dynamicQuery = Wrappers.<DemandLibrary>lambdaQuery();
        dynamicQuery.eq(DemandLibrary::getCreateUserId,user.getId());
        dynamicQuery.eq(DemandLibrary::getDeleted,0);
        dynamicQuery.isNull(DemandLibrary::getSubProjectId);
        if(StringUtils.isNotBlank(demandLibraryQuery.getKey())){
            //需求编号
            dynamicQuery.like(DemandLibrary::getDemandNumber,demandLibraryQuery.getKey());
            //需求名称
            LambdaQueryWrapper<DemandLibrary> or1 = dynamicQuery.or();
            or1.like(DemandLibrary::getDemandName,demandLibraryQuery.getKey());
            //需求内容
            LambdaQueryWrapper<DemandLibrary> or2 = or1.or();
            or2.like(DemandLibrary::getDemandContent,demandLibraryQuery.getKey());
            //提交单位
            LambdaQueryWrapper<DemandLibrary> or3 = or2.or();
            or3.like(DemandLibrary::getUnitName,demandLibraryQuery.getKey());
            //提交人
            LambdaQueryWrapper<DemandLibrary> or4 = or3.or();
            or4.like(DemandLibrary::getSubmitter,demandLibraryQuery.getKey());
        }

        dynamicQuery.orderByDesc(DemandLibrary::getSubmitTime);

        Page<DemandLibrary> all = demandLibraryMap.selectPage(new Page<>(page, size),dynamicQuery);
        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandLibraryPageQueryRes dicPageQueryRes = DemandLibraryPageQueryRes
                            .builder()
                            .id(element.getId())
                            .demandNumber(element.getDemandNumber())
                            .demandName(element.getDemandName())
                            .demandContent(element.getDemandContent())
                            .demandClass(dictionaryMapper.selectById(element.getDemandClassId()).getValue())
                            .unitName(element.getUnitName())
                            .demandStatus(element.getDemandStatus() == -1 ? "废除"
                                    : element.getDemandStatus() == 0 ? "立项" : "认领")
                            .demandSchedule(element.getDemandSchedule())
                            .associatedProjectId(projectMapper.selectById(element.getAssociatedProjectId()).getName())
                            .submitter(element.getSubmitter())
                            .submitTime(element.getSubmitTime())
                            .deleted(element.getDeleted())
                            .build();
                    return dicPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<DemandLibraryQueryRes> findById(String id) {
        Integer idInt = Integer.parseInt(id);
        DemandLibrary demandLibrary = demandLibraryMap.selectById(idInt);

        DemandLibraryQueryRes demandLibraryQueryRes = EntityUtils.copyProperties(demandLibrary,DemandLibraryQueryRes.class);
        demandLibraryQueryRes.setDemandClass(dictionaryMapper.selectById(demandLibrary.getDemandClassId()).getValue());
        demandLibraryQueryRes.setDemandStatus(demandLibrary.getDemandStatus() == -1 ? "废除"
                : demandLibrary.getDemandStatus() == 0 ? "立项" : "认领");
        demandLibraryQueryRes.setAssociatedProjectId(projectMapper.selectById(demandLibrary.getAssociatedProjectId()).getName());

        //附件
        LambdaQueryWrapper<DemandAttachment> demands = new LambdaQueryWrapper<> ();
        demands.eq(DemandAttachment::getDemandId,idInt);
        demands.eq(DemandAttachment::getDeleted,0);
        List<DemandAttachment> demandAttachmentList = demandAttachmentMapper.selectList(demands);


        List<DemandAttachmentRes> dicList = new ArrayList<> ();
        for(DemandAttachment dic:demandAttachmentList){
            DemandAttachmentRes demandAttachmentRes = new DemandAttachmentRes();
            demandAttachmentRes.setId(dic.getId());
            demandAttachmentRes.setSubmitter(userMapper.selectById(fileInfoMapper.selectById(dic.getFileId()).getCreateUserId()).getName());
            demandAttachmentRes.setCreateTime(fileInfoMapper.selectById(dic.getFileId()).getCreateTime());
            demandAttachmentRes.setName(fileInfoMapper.selectById(dic.getFileId()).getName());
            dicList.add(demandAttachmentRes);
        }
        demandLibraryQueryRes.setDemandA(dicList);

        //子项目
        LambdaQueryWrapper<DemandLibrary> demandLibraryMap1 = new LambdaQueryWrapper<>();
        demandLibraryMap1.eq(DemandLibrary::getSubProjectId,idInt);
        demandLibraryMap1.eq(DemandLibrary::getDeleted,0);
        List<DemandLibrary> demandLibraryList = demandLibraryMap.selectList(demandLibraryMap1);
        demandLibraryQueryRes.setDemandB(demandLibraryList);

        return ResponseBuilder.ok(demandLibraryQueryRes);
    }


}
