package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.migao.config.constant.ExamineStatus;
import com.migao.config.constant.FinancesStatus;
import com.migao.config.constant.ProjectStatus;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.Milestone;
import com.migao.entity.po.Project;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.DictionaryListRes;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.mapper.DictionaryMapper;
import com.migao.mapper.MilestoneMapper;
import com.migao.mapper.ProjectMapper;
import com.migao.mapper.ProjectRepository;
import com.migao.service.ProjectService;
import com.migao.util.EntityUtils;
import com.migao.util.ProjectNoUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import javax.annotation.Resource;
import javax.validation.constraints.NotEmpty;
import java.util.List;

@Service
@Transactional(rollbackFor = {Exception.class})
public class ProjectServiceImpl implements ProjectService {

    @Resource
    private ProjectRepository projectRepository;

    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private DictionaryMapper dictionaryMapper;
    @Resource
    private MilestoneMapper milestoneMapper;


    /**
     * 增
     *
     * @param projectInsertReq projectInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(ProjectInsertReq projectInsertReq) {
        Project project = new Project();
        BeanUtils.copyProperties(projectInsertReq, project);
        project.setReserveStatus(ProjectStatus.TO_APPROVAL.getName());
        project.setDeleted(0);
        project.setParentId(0);
        project.setNo(ProjectNoUtil.getCode());
        return ResponseBuilder.ok(projectRepository.save(project));
    }

    /**
     * 分页查
     *
     * @param projectPageQueryReq projectPageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(ProjectPageQueryReq projectPageQueryReq) {
        //1。分页查询所有供公司
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (projectPageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, projectPageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, projectPageQueryReq.getOrderStatus());
        }
        queryWrapper.eq(Project::getReserveStatus, ProjectStatus.TO_APPROVAL.getName());
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    /**
     * 删
     *
     * @param id id
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        projectRepository.findById(id).ifPresent(
                i -> {
                    i.setDeleted(1);
                    projectRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 批量删
     *
     * @param ids ids
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
        projectRepository.findAllById(ids).forEach(
                i -> {
                    i.setDeleted(1);
                    projectRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 改
     *
     * @param projectUpdateReq projectUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(ProjectUpdateReq projectUpdateReq) {
        projectRepository.save(EntityUtils.copyProperties(projectUpdateReq, Project.class));
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<List<ProjectQueryRes>> query(ProjectQueryReq projectQueryReq) {
        return null;
    }


    @Override
    public ResponseBean<List<DictionaryListRes>> projectClassify() {
        String value = "PROJECT_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<List<DictionaryListRes>> specialtyClassify() {
        String value = "SPECIALTY_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<List<DictionaryListRes>> twoClassify() {
        String value = "TWO_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<ProjectQueryRes> findById(Integer id) {
        Project project = projectMapper.selectById(id);
        ProjectQueryRes projectQueryRes = new ProjectQueryRes();
        BeanUtils.copyProperties(project, projectQueryRes);
        return ResponseBuilder.ok(projectQueryRes);
    }

    @Override
    public ResponseBean<?> approve(Integer id, ExamineStatus status) {
        Project project = projectMapper.selectById(id);
        if (project == null) {
            ResponseBuilder.fail("项目不存在");
        }
        if (status.equals(FinancesStatus.PASS)){
            project.setOrderStatus(ProjectStatus.RESERVES_ISSUED.getName());
            project.setReserveStatus(ProjectStatus.APPROVE.getName());
            projectMapper.updateById(project);
        }else {
            project.setReserveStatus(ProjectStatus.REJECT.getName());
            projectMapper.updateById(project);
        }

        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> importData(String fileId) {
        return null;
    }

    @Override
    public ResponseBean<?> export(ProjectPageQueryReq projectPageQueryReq) {
        return null;
    }

    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQueryControl(ProjectPageQueryReq projectPageQueryReq) {
        //1。分页查询所有供公司
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (projectPageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, projectPageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, projectPageQueryReq.getOrderStatus());
        }
        queryWrapper.eq(Project::getOrderStatus, ProjectStatus.RESERVES_ISSUED.getName());
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<?> insertSubProject(SubProjectInsertReq subProjectInsertReq) {
        //查询父级项目
        Project parProject = projectMapper.selectById(subProjectInsertReq.getParentId());
        if (parProject==null){
            return ResponseBuilder.fail("父级项目不存在");
        }
        //创建并关联父级项目
        Project project = new Project();
        BeanUtils.copyProperties(subProjectInsertReq, project);
        project.setReserveStatus(ProjectStatus.APPROVE.getName());
        project.setOrderStatus(ProjectStatus.TO_BE_REPORTED.getName());
        project.setDeleted(0);
        project.setParentId(subProjectInsertReq.getParentId());
        project.setNo(ProjectNoUtil.getCode());
        Project save = projectRepository.save(project);

        //更新项目下达状态
        project.setOrderStatus(ProjectStatus.APPROVE.getName());
        parProject.setPurchaseStatus(ProjectStatus.TO_BE_REPORTED.getName());
        projectMapper.updateById(parProject);
        return ResponseBuilder.ok();
    }
}
